import java.util.*;

public class Clock extends Thread {
    public static int clockCalibration = 1000 ; // How much does 1s last in ms
    private int seconds;
    private int clockReset;
    private int bloodTest;
    private int hardwareTest;
    private int alarmCheck;

    //Variables for multiple message displaying
    private String messages[] = new String[5];
    private int messagePointer = 0;
    private int outputPointer = 0;
    private int arraySize = 0;

    //Constructor
    public Clock() {
	//Set default values for finished working software
	seconds = 0;
	bloodTest = 600;
	hardwareTest = 30;
	alarmCheck = 10;

	//Intialise Message buffer
	for (int i = 0; i < 5 ; i++) {
	    messages[i] = "";
	}
    }

    //Constructor
    public Clock(int bloodTestValue, int hardwareTestValue, int alarmCheckValue) {
	//Allow values to be set, to speed up the simulation
	seconds = 0;
	bloodTest = bloodTestValue;
	hardwareTest = hardwareTestValue;
	alarmCheck = alarmCheckValue;

	//Intialise Message buffer
	for (int i = 0; i < 5 ; i++) {
	    messages[i] = "";
	}
    }

    private boolean running    = false ;

    public synchronized boolean isRunning() {
	return running ;
    } // EndMethod isRunning

    private synchronized void setRunning(boolean newRunningState) {
	running = newRunningState ;
    } // EndMethod setRunning

    public synchronized void stopClockBlocking() {
	setRunning(false) ;
    } // EndMethod stopClock

    public void run() {
	// Simulate clock timer
	setRunning(true) ;
	while (isRunning()) {
	    synchronized(this) {
		// Reset the clock every 24 hours (86400 seconds in 24 hours)
		if (seconds == 86400) {
		    seconds = 0;
		    Controller.resetCumulativeDose();
		}

		// Every 10 mins (600 seconds) trigger controller to
		// determine whether any insulin should be admiinistered
		if ((seconds % bloodTest) == 0) {
		    Controller.changeState(2); //State 2 = the run state
		}

		// Every 30 seconds run hardware test
		if ((seconds % hardwareTest) == 0) {
		    Controller.changeState(4); //State 4 = the test state
		}

		// Every 10 seconds check the alarm
		if ((seconds % alarmCheck) == 0) {
		    Controller.changeState(5); //State 5 = the alarm check state
		}
	
		if ((seconds % 5) == 0) {
		    Display.displayOutput(messages[outputPointer]);

		    //Select next message to be output
		    if (outputPointer < arraySize -1) {
			outputPointer = outputPointer + 1;
		    }
		    else {
			outputPointer = 0;
		    }
		}		
	    } // EndSynchronized

	    // wait 1000 milliseconds (1 second) by default
	    try {
		sleep(Clock.clockCalibration);
	    }
	    catch (InterruptedException ie) {}

	    synchronized(this) {

		// Increase the clock by 1 second
		seconds = seconds + 1;
			
		//Update the clock displayed to the user
		Controller.updateClock(seconds);
		if (GenericTestBed.getActiveTestBedObject()!=null)
		    GenericTestBed
			.getActiveTestBedObject()
			.aNewSecondHasPassed() ;

	    } // EndSynchronized
	}
	
    } // EndMethod run

    public synchronized void displayOutput(String output) {
	messages[messagePointer] = output;

	//Increment messagePointer
	if (messagePointer < 4) {
	    messagePointer = messagePointer + 1;
	}
	else {
	    messagePointer = 0;
	}

	if (arraySize < 4) {
	    arraySize = arraySize + 1;
	}
    }

    public synchronized int getClockSeconds() {
	return seconds ;
    } // EndMethod getClockSeconds
}
