/**
 * HardwareSimulatorAPI.java
 * 
 * Time-stamp: <2005-02-11 18:28:39 ftaiani>
 * @author Francois Taiani   <f.taiani@computer.org>
 * $Id: HardwareSimulatorAPI.java,v 1.1 2005/02/11 19:42:29 ftaiani Exp $
 * 
 */

/**
 * This is a facade class that offers a number of methos to simulate a
 * number of hardware events.
 *
 * @author Francois Taiani   <f.taiani@computer.org>
 */
public class HardwareSimulatorAPI {

    private GenericTestBed myTestBed ;

    public HardwareSimulatorAPI(GenericTestBed aTestBed) {
	myTestBed = aTestBed ;
    } // EndConstructor HardwareSimulatorAPI

    public void startInsulinPumpController() {
	myTestBed.printMessage("startInsulinPumpController");
	Controller.startClockBlocking(10, 10, 10);
    } // EndMethod startInsulinPump

    public void stopInsulinPumpController() {
	myTestBed.printMessage("stopInsulinPumpController");
	Controller.stopClock() ;
    } // EndMethod stopInsulinPumpController

    public void removeInsulinReservoir() {
	myTestBed.printMessage("removeInsulinReservoir");
	InsulinReservoir.removeInsulin();
    } // EndMethod removeInsulinReservoir

    public void replaceInsulinReservoir() {
	myTestBed.printMessage("replaceInsulinReservoir");
	InsulinReservoir.replaceInsulin();
    } // EndMethod replaceInsulinReservoir

    public boolean insulinReservoirIsPresent() {
	return InsulinReservoir.isInsulinPresent();
    }

    public void removeNeedle() {
	myTestBed.printMessage("removeNeedle");
	NeedleAssembly.setNeedlePresent(false);
    } // EndMethod removeNeedle

    public void replaceNeedle() {
	myTestBed.printMessage("replaceNeedle");
	NeedleAssembly.setNeedlePresent(true);
    } // EndMethod replaceNeedle

    public boolean needleIsPresent() {
	return NeedleAssembly.isNeedlePresent() ;
    } // EndMethod needleIsPresent

    public void setBatteryAtLowLevel() {
	myTestBed.printMessage("setBatteryAtLowLevel");
	PowerSupply.setBatteryLevel(false);
    } // EndMethod setBatteryAtLowLevel

    public void replaceBattery() {
	myTestBed.printMessage("replaceBattery");
	PowerSupply.setBatteryLevel(false);
    } // EndMethod replaceBattery

    public boolean batteryIsAtLowLevel() {
	return PowerSupply.isBatteryLevelOK();
    } // EndMethod batteryIsAtLowLevel

    public void simulateSugarSensorFailure() {
	myTestBed.printMessage("simulateSugarSensorFailure");
	Sensor.setSensorWorking(false);
    } // EndMethod simulateSugarSensorFailure

    public void repairSugarSensor() {
	myTestBed.printMessage("repairSugarSensor");
	Sensor.setSensorWorking(true);
    } // EndMethod repairSugarSensor

    public boolean sensorIsWorking() {
	return Sensor.isSensorWorking() ;
    } // EndMethod sensorIsWorking

    public void simulatePumpFailure() {
	myTestBed.printMessage("simulatePumpFailure");
	InsulinPump.setPumpWorking(false);
    } // EndMethod simulatePumpFailure

    public void repairPump() {
	myTestBed.printMessage("repairPump");
	InsulinPump.setPumpWorking(true);
    } // EndMethod repairPump

    public boolean pumpIsWorking() {
	return InsulinPump.isPumpWorking();
    } // EndMethod pumpIsWorking

    public void simulateDeliveryFailure() {
	myTestBed.printMessage("simulateDeliveryFailure");
	Controller.setDeliveryWorking(false);
    } // EndMethod simulateDeliveryFailure

    public void repairDelivery() {
	myTestBed.printMessage("repairDelivery");
	Controller.setDeliveryWorking(true);
    } // EndMethod repairDelivery

    public boolean deliveryIsWorking() {
	return Controller.isDeliveryWorking();
    } // EndMethod isDeliveryWorking

    public void setSugarNewSensorReading(int newBloodSugarLevel) {
	myTestBed.printMessage("setSugarNewSensorReading("
			       +newBloodSugarLevel+")");
	Sensor.setReading(newBloodSugarLevel);
    } // EndMethod setNewSugarSensorReading

} // EndClass HardwareSimulatorAPI
